<?
/**
 *
 * 12/08/2017 - criado por thiago.farias
 *
 */
class PenAtualizarSeiRN extends PenAtualizadorRN {

    const PARAMETRO_VERSAO_MODULO_ANTIGO = 'PEN_VERSAO_MODULO_SEI';
    const PARAMETRO_VERSAO_MODULO = 'VERSAO_MODULO_PEN';

    public function __construct() {
        parent::__construct();
    }

    protected function inicializarObjInfraIBanco(){
        return BancoSEI::getInstance();
    }

    protected function atualizarVersaoConectado() {
        try {
            $this->inicializar('INICIANDO ATUALIZACAO DO MODULO PEN NO SEI VERSAO ' . SEI_VERSAO);

            //testando se esta usando BDs suportados
            if (!(BancoSEI::getInstance() instanceof InfraMySql) &&
                !(BancoSEI::getInstance() instanceof InfraSqlServer) &&
                !(BancoSEI::getInstance() instanceof InfraOracle)) {

                $this->finalizar('BANCO DE DADOS NAO SUPORTADO: ' . get_parent_class(BancoSEI::getInstance()), true);
            }

            PENIntegracao::validarCompatibilidadeModulo();

            //testando permissoes de criaes de tabelas
            $objInfraMetaBD = new InfraMetaBD(BancoSEI::getInstance());

            if (count($objInfraMetaBD->obterTabelas('pen_sei_teste')) == 0) {
                BancoSEI::getInstance()->executarSql('CREATE TABLE pen_sei_teste (id ' . $objInfraMetaBD->tipoNumero() . ' null)');
            }
            BancoSEI::getInstance()->executarSql('DROP TABLE pen_sei_teste');

            $objInfraParametro = new InfraParametro(BancoSEI::getInstance());

            // Aplicao de scripts de atualizao de forma incremental
            // Ausncia de [break;] proposital para realizar a atualizao incremental de verses
            $strVersaoModuloPen = $objInfraParametro->getValor(self::PARAMETRO_VERSAO_MODULO, false) ?: $objInfraParametro->getValor(self::PARAMETRO_VERSAO_MODULO_ANTIGO, false);
            switch ($strVersaoModuloPen) {
                case '':      $this->instalarV100(); // Nenhuma verso instalada
                case '1.0.0': $this->instalarV101();
                case '1.0.1': $this->instalarV110();
                case '1.1.0': $this->instalarV111();
                case '1.1.1': //No houve atualizao no banco de dados
                case '1.1.2': //No houve atualizao no banco de dados
                case '1.1.3': //No houve atualizao no banco de dados
                case '1.1.4': //No houve atualizao no banco de dados
                case '1.1.5': //No houve atualizao no banco de dados
                case '1.1.6': $this->instalarV117();
                case '1.1.7': $this->instalarV118();
                case '1.1.8': $this->instalarV119();
                case '1.1.9': $this->instalarV1110();
                case '1.1.10': $this->instalarV1111();
                case '1.1.11': $this->instalarV1112();
                case '1.1.12': $this->instalarV1113();
                case '1.1.13': $this->instalarV1114();
                case '1.1.14': $this->instalarV1115();
                case '1.1.15': $this->instalarV1116();
                case '1.1.16': $this->instalarV1117();
                case '1.1.17': $this->instalarV1200();
                case '1.2.0': $this->instalarV1201();
                case '1.2.1': $this->instalarV1202();
                case '1.2.2': $this->instalarV1203();

                break;
                default:
                $this->finalizar('VERSAO DO MDULO J CONSTA COMO ATUALIZADA');
                break;
            }

            $this->finalizar('FIM');
            InfraDebug::getInstance()->setBolDebugInfra(true);
        } catch (Exception $e) {
            InfraDebug::getInstance()->setBolLigado(false);
            InfraDebug::getInstance()->setBolDebugInfra(false);
            InfraDebug::getInstance()->setBolEcho(false);
            throw new InfraException('Erro atualizando VERSAO.', $e);
        }
    }

    /**
     * Cria um novo parmetro
     * @return int Cdigo do Parametro gerado
     */
    protected function criarParametro($strNome, $strValor, $strDescricao) {
        $objDTO = new PenParametroDTO();
        $objDTO->setStrNome($strNome);
        $objDTO->setStrValor($strValor);
        $objDTO->setStrDescricao($strDescricao);
        $objDTO->retStrNome();

        $objBD = new PenParametroBD(BancoSEI::getInstance());
        $objDTOCadastrado = $objBD->cadastrar($objDTO);

        return $objDTOCadastrado->getStrNome();
    }

    /**
     * Remove um parmetro do infra_parametros
     * @return string Nome do parmetro
     */
    protected function deletaParametroInfra($strNome) {
        $objDTO = new InfraParametroDTO();
        $objDTO->setStrNome($strNome);

        $objBD = new InfraParametroBD(BancoSEI::getInstance());
        return $objBD->excluir($objDTO);
    }

    /* Contm atualizaes da versao 1.0.0 do modulo */
    protected function instalarV100() {

        $objInfraBanco = BancoSEI::getInstance();
        $objMetaBD = $this->objMeta;

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_processo_eletronico',
            'cols' => array(
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_procedimento' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('numero_registro')),
            'uk' => array('numero_registro', 'id_procedimento'),
            'fks' => array(
                'procedimento' => array('nome' => 'fk_md_pen_proc_eletr_procedim',
                    'cols' => array('id_procedimento', 'id_procedimento')),
            )
        ));


        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_tramite',
            'cols' => array(
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'ticket_envio_componentes' => array($objMetaBD->tipoTextoGrande(), PenMetaBD::SNULLO),
                'dth_registro' => array($objMetaBD->tipoDataHora(), PenMetaBD::SNULLO),
                'id_andamento' => array($objMetaBD->tipoNumero(), PenMetaBD::SNULLO),
                'id_usuario' => array($objMetaBD->tipoNumero(), PenMetaBD::SNULLO),
                'id_unidade' => array($objMetaBD->tipoNumero(), PenMetaBD::SNULLO)
            ),
            'pk' => array('cols'=>array('id_tramite')),
            'uk' => array('numero_registro', 'id_tramite'),
            'fks' => array(
                'md_pen_processo_eletronico' => array('nome'=>'fk_md_pen_tramite_proc_eletr',
                  'cols' => array('numero_registro', 'numero_registro')),
                'usuario' => array('id_usuario', 'id_usuario'),
                'unidade' => array('id_unidade', 'id_unidade'),
            )
        ));


        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_especie_documental',
            'cols' => array(
                'id_especie' => array($objMetaBD->tipoNumero(16), PenMetaBD::NNULLO),
                'nome_especie' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO),
                'descricao' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('id_especie')),
        ));


        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_tramite_pendente',
            'cols' => array(
                'id' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'numero_tramite' => array($objMetaBD->tipoTextoVariavel(255)),
                'id_atividade_expedicao' => array($objMetaBD->tipoNumero(), PenMetaBD::SNULLO)
            ),
            'pk' => array('cols'=>array('id')),
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_tramite_recibo_envio',
            'cols' => array(
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'dth_recebimento' => array($objMetaBD->tipoDataHora(), PenMetaBD::NNULLO),
                'hash_assinatura' => array($objMetaBD->tipoTextoVariavel(345), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('numero_registro', 'id_tramite')),
        ));


        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_procedimento_andamento',
            'cols' => array(
                'id_andamento' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'id_procedimento' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'situacao' => array($objMetaBD->tipoTextoFixo(1), 'N'),
                'data' => array($objMetaBD->tipoDataHora(), PenMetaBD::NNULLO),
                'mensagem' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO),
                'hash' => array($objMetaBD->tipoTextoFixo(32), PenMetaBD::NNULLO),
                'id_tarefa' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO)
            ),
            'pk' => array('nome' => 'pk_md_pen_procedim_andamen', 'cols'=>array('id_andamento')),
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_protocolo',
            'cols' => array(
                'id_protocolo' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'sin_obteve_recusa' => array($objMetaBD->tipoTextoFixo(1), 'N')
            ),
            'pk' => array('cols'=>array('id_protocolo')),
            'fks' => array(
                'protocolo' => array('id_protocolo', 'id_protocolo')
            )
        ));


        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_recibo_tramite',
            'cols' => array(
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'dth_recebimento' => array($objMetaBD->tipoDataHora(), PenMetaBD::NNULLO),
                'hash_assinatura' => array($objMetaBD->tipoTextoVariavel(345), PenMetaBD::NNULLO),
                'cadeia_certificado' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('numero_registro', 'id_tramite')),
            'fks' => array(
                'md_pen_tramite' => array('nome' => 'fk_md_pen_rec_tramite_tramite',
                  'cols' => array(array('numero_registro', 'id_tramite'), array('numero_registro', 'id_tramite')))
            )
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_recibo_tramite_enviado',
            'cols' => array(
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'dth_recebimento' => array($objMetaBD->tipoDataHora(), PenMetaBD::NNULLO),
                'hash_assinatura' => array($objMetaBD->tipoTextoVariavel(345), PenMetaBD::NNULLO),
                'cadeia_certificado ' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO)
            ),
            'pk' => array('nome' => 'pk_md_pen_recibo_tram_envia', 'cols'=>array('numero_registro', 'id_tramite')),
            'fks' => array(
                'md_pen_tramite' => array('nome' => 'fk_md_pen_rec_tram_env_tram',
                  'cols' => array(array('numero_registro', 'id_tramite'), array('numero_registro', 'id_tramite')))
            )
        ));


        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_recibo_tramite_recebido',
            'cols' => array(
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'dth_recebimento' => array($objMetaBD->tipoDataHora(), PenMetaBD::NNULLO),
                'hash_assinatura' => array($objMetaBD->tipoTextoVariavel(345), PenMetaBD::NNULLO)
            ),
            'pk' => array('nome' => 'pk_md_pen_recibo_tramite_receb', 'cols'=>array('numero_registro', 'id_tramite', 'hash_assinatura')),
            'fks' => array(
                'md_pen_tramite' => array('nome' => 'fk_md_pen_recibo_receb_tram',
                  'cols' => array(array('numero_registro', 'id_tramite'), array('numero_registro', 'id_tramite')))
            )
        ));


        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_rel_processo_apensado',
            'cols' => array(
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_procedimento_apensado' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'protocolo' => array($objMetaBD->tipoTextoVariavel(50), PenMetaBD::NNULLO)
            ),
            'pk' => array('nome' => 'pk_md_pen_rel_processo_apensad', 'cols'=>array('numero_registro', 'id_procedimento_apensado')),
            'fks' => array(
                'md_pen_processo_eletronico' => array('nome' => 'fk_md_pen_proc_eletr_apensado',
                  'cols' => array('numero_registro', 'numero_registro'))
            )
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_rel_serie_especie',
            'cols' => array(
                'codigo_especie' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'id_serie' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'sin_padrao' => array($objMetaBD->tipoTextoFixo(1), 'N')
            ),
            'pk' => array('cols'=>array('id_serie')),
            'uk' => array('codigo_especie', 'id_serie'),
            'fks' => array(
                'serie' =>  array('nome' => ' fk_md_pen_rel_serie_especie', 'cols' => array('id_serie', 'id_serie'))
            )
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_rel_tarefa_operacao',
            'cols' => array(
                'id_tarefa' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'codigo_operacao' => array($objMetaBD->tipoTextoFixo(2), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('id_tarefa', 'codigo_operacao')),
            'fks' => array(
                'tarefa' =>  array('nome' => 'fk_md_pen_rel_operacao_tarefa', 'cols' => array('id_tarefa', 'id_tarefa'))
            )
        ));


        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_rel_tipo_doc_map_rec',
            'cols' => array(
                'codigo_especie' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'id_serie' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'sin_padrao' => array($objMetaBD->tipoTextoFixo(2), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('codigo_especie', 'id_serie')),
            'fks' => array(
                'serie' => array('nome' =>'fk_md_pen_rel_tipo_doc_serie', 'cols' => array('id_serie', 'id_serie'))
            )
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_componente_digital',
            'cols' => array(
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_procedimento' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'id_documento' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'id_anexo' => array($objMetaBD->tipoNumero(), PenMetaBD::SNULLO),
                'protocolo' => array($objMetaBD->tipoTextoVariavel(50), PenMetaBD::NNULLO),
                'nome' => array($objMetaBD->tipoTextoVariavel(100), PenMetaBD::NNULLO),
                'hash_conteudo' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO),
                'algoritmo_hash' => array($objMetaBD->tipoTextoVariavel(20), PenMetaBD::NNULLO),
                'tipo_conteudo' => array($objMetaBD->tipoTextoFixo(3), PenMetaBD::NNULLO),
                'mime_type' => array($objMetaBD->tipoTextoVariavel(100), PenMetaBD::NNULLO),
                'dados_complementares' => array($objMetaBD->tipoTextoVariavel(1000), PenMetaBD::SNULLO),
                'tamanho' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'ordem' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'sin_enviar' => array($objMetaBD->tipoTextoFixo(1), 'N')
            ),
            'pk' => array('cols'=>array('numero_registro', 'id_procedimento', 'id_documento', 'id_tramite')),
            'fks' => array(
                'anexo' => array('nome' => 'fk_md_pen_comp_dig_anexo', 'cols' => array('id_anexo', 'id_anexo')),
                'documento' => array('nome' => 'fk_md_pen_comp_dig_documento', 'cols' => array('id_documento', 'id_documento')),
                'procedimento' => array('nome' => 'fk_md_pen_comp_dig_procediment', 'cols' => array('id_procedimento', 'id_procedimento')),
                'md_pen_processo_eletronico' => array('nome' => 'fk_md_pen_comp_dig_proc_eletr', 'cols' => array('numero_registro', 'numero_registro')),
                'md_pen_tramite' => array('nome' => 'fk_md_pen_comp_dig_tramite', 'cols' => array(array('numero_registro', 'id_tramite'), array('numero_registro', 'id_tramite')))
            )
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_unidade',
            'cols' => array(
                'id_unidade' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'id_unidade_rh' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('id_unidade')),
            'fks' => array(
                'unidade' => array('id_unidade', 'id_unidade')
            )
        ));


        //----------------------------------------------------------------------
        // Novas sequncias
        //----------------------------------------------------------------------
        $objInfraSequencia = new InfraSequencia($objInfraBanco);

        if (!$objInfraSequencia->verificarSequencia('md_pen_procedimento_andamento')) {
            $objInfraSequencia->criarSequencia('md_pen_procedimento_andamento', '1', '1', '9999999999');
        }

        if (!$objInfraSequencia->verificarSequencia('md_pen_tramite_pendente')) {
            $objInfraSequencia->criarSequencia('md_pen_tramite_pendente', '1', '1', '9999999999');
        }

        //----------------------------------------------------------------------
        // Parmetros
        //----------------------------------------------------------------------

        $objInfraParametro = new InfraParametro($objInfraBanco);

        $objInfraParametro->setValor('PEN_ID_REPOSITORIO_ORIGEM', '');
        $objInfraParametro->setValor('PEN_ENDERECO_WEBSERVICE', '');
        $objInfraParametro->setValor('PEN_SENHA_CERTIFICADO_DIGITAL', '1234');
        $objInfraParametro->setValor('PEN_TIPO_PROCESSO_EXTERNO', '100000320');
        $objInfraParametro->setValor('PEN_ENVIA_EMAIL_NOTIFICACAO_RECEBIMENTO', 'N');
        $objInfraParametro->setValor('PEN_ENDERECO_WEBSERVICE_PENDENCIAS', '');
        $objInfraParametro->setValor('PEN_UNIDADE_GERADORA_DOCUMENTO_RECEBIDO', '');
        $objInfraParametro->setValor('PEN_LOCALIZACAO_CERTIFICADO_DIGITAL', '');

        //----------------------------------------------------------------------
        // Especie de Documento
        //----------------------------------------------------------------------

        $objBD = new GenericoBD(BancoSEI::getInstance());
        $objDTO = new EspecieDocumentalDTO();

        $fnCadastrar = function($dblIdEspecie, $strNomeEspecie, $strDescricao) use($objDTO, $objBD) {

            $objDTO->unSetTodos();
            $objDTO->setStrNomeEspecie($strNomeEspecie);

            if ($objBD->contar($objDTO) == 0) {
                $objDTO->setDblIdEspecie($dblIdEspecie);
                $objDTO->setStrDescricao($strDescricao);
                $objBD->cadastrar($objDTO);
            }
        };

        $fnCadastrar(1, 'Abaixo-assinado', 'Podendo ser complementado: de Reivindicao');
        $fnCadastrar(2, 'Acrdo', 'Expressa deciso proferida pelo Conselho Diretor, no abrangida pelos demais instrumentos deliberativos anteriores.');
        $fnCadastrar(3, 'Acordo', 'Podendo ser complementado: de Nvel de Servio; Coletivo de Trabalho');
        $fnCadastrar(4, 'Alvar', 'Podendo ser complementado: de Funcionamento; Judicial');
        $fnCadastrar(5, 'Anais', 'Podendo ser complementado: de Eventos; de Engenharia');
        $fnCadastrar(6, 'Anteprojeto', 'Podendo ser complementado: de Lei');
        $fnCadastrar(7, 'Aplice', 'Podendo ser complementado: de Seguro');
        $fnCadastrar(8, 'Apostila', 'Podendo ser complementado: de Curso');
        $fnCadastrar(9, 'Ata', 'Como Documento Externo pode ser complementado: de Reunio; de Realizao de Prego');
        $fnCadastrar(10, 'Atestado', 'Podendo ser complementado: Mdico; de Comparecimento; de Capacidade Tcnica');
        $fnCadastrar(11, 'Ato', 'Expressa deciso sobre outorga, expedio, modificao, transferncia, prorrogao, adaptao e extino de concesses, permisses e autorizaes para explorao de servios, uso de recursos escassos e explorao de satlite, e Chamamento Pblico.');
        $fnCadastrar(12, 'Auto', 'Podendo ser complementado: de Vistoria; de Infrao');
        $fnCadastrar(13, 'Aviso', 'Podendo ser complementado: de Recebimento; de Sinistro; de Frias');
        $fnCadastrar(14, 'Balancete', 'Podendo ser complementado: Financeiro');
        $fnCadastrar(15, 'Balano', 'Podendo ser complementado: Patrimonial - BP; Financeiro');
        $fnCadastrar(16, 'Bilhete', 'Podendo ser complementado: de Pagamento; de Loteria');
        $fnCadastrar(17, 'Boletim', 'Podendo ser complementado: de Ocorrncia; Informativo');
        $fnCadastrar(18, 'Carta', 'Podendo ser complementado: Convite');
        $fnCadastrar(19, 'Cartaz', 'Podendo ser complementado: de Evento');
        $fnCadastrar(20, 'Cdula', 'Podendo ser complementado: de Identidade; de Crdito Bancrio; de Crdito Comercial; de Crdito Imobilirio');
        $fnCadastrar(21, 'Certido', 'Como Documento Externo pode ser complementado: de Tempo de Servio; de Nascimento; de Casamento; de bito; Negativa de Falncia ou Concordata; Negativa de Dbitos Trabalhistas; Negativa de Dbitos Tributrios');
        $fnCadastrar(22, 'Certificado', 'Podendo ser complementado: de Concluso de Curso; de Calibrao de Equipamento; de Marca');
        $fnCadastrar(23, 'Cheque', 'Podendo ser complementado: Cauo');
        $fnCadastrar(24, 'Comprovante', 'Podendo ser complementado: de Despesa; de Rendimento; de Residncia; de Matrcula; de Unio Estvel');
        $fnCadastrar(25, 'Comunicado', 'Expediente interno entre uma unidade administrativa e um servidor ou entre um servidor e uma unidade administrativa de um mesmo rgo pblico.');
        $fnCadastrar(26, 'Consulta', 'Podendo ser complementado: Pblica; Interna');
        $fnCadastrar(27, 'Contracheque', 'Espcie prpria');
        $fnCadastrar(28, 'Contrato', 'Como Documento Externo pode ser complementado: Social');
        $fnCadastrar(29, 'Convnio', 'Espcie prpria');
        $fnCadastrar(30, 'Convite', 'Podendo ser complementado: de Reunio; para Evento; de Casamento');
        $fnCadastrar(31, 'Conveno', 'Podendo ser complementado: Coletiva de Trabalho; Internacional');
        $fnCadastrar(32, 'Crach', 'Podendo ser complementado: de Identificao; de Evento');
        $fnCadastrar(33, 'Cronograma', 'Podendo ser complementado: de Projeto; de Estudos');
        $fnCadastrar(34, 'Currculo', 'Podendo ser complementado: de Candidato');
        $fnCadastrar(35, 'Debnture', 'Espcie prpria');
        $fnCadastrar(36, 'Deciso', 'Podendo ser complementado: Administrativa; Judicial');
        $fnCadastrar(37, 'Declarao', 'Como Documento Externo pode ser complementado: de Imposto de Renda; de Conformidade; de Responsabilidade Tcnica; de Acumulao de Aposentadoria; de Acumulao de Cargos; de Informaes Econmico-Fiscais da Pessoa Jurdica $fnCadastrar(DIPJ);');
        $fnCadastrar(38, 'Decreto', 'Espcie prpria');
        $fnCadastrar(39, 'Deliberao', 'Podendo ser complementado: de Recursos; do Conselho');
        $fnCadastrar(40, 'Demonstrativo', 'Podendo ser complementado: Financeiro; de Pagamento; de Arrecadao');
        $fnCadastrar(41, 'Depoimento', 'Podendo ser complementado: das Testemunhas');
        $fnCadastrar(42, 'Despacho', 'Espcie prpria');
        $fnCadastrar(43, 'Dirio', 'Podendo ser complementado: de Justia; Oficial');
        $fnCadastrar(44, 'Diploma', 'Podendo ser complementado: de Concluso de Curso');
        $fnCadastrar(45, 'Diretriz', 'Podendo ser complementado: Oramentria');
        $fnCadastrar(46, 'Dissertao', 'Podendo ser complementado: de Mestrado');
        $fnCadastrar(47, 'Dossi', 'Podendo ser complementado: de Processo; Tcnico');
        $fnCadastrar(48, 'Edital', 'Podendo ser complementado: de Convocao; de Intimao; de Lanamento');
        $fnCadastrar(49, 'E-mail', 'Indicado nos Parmetros para corresponder ao envio de Correspondncia Eletrnica do SEI');
        $fnCadastrar(50, 'Embargos', 'Podendo ser complementado: de Declarao; de Execuo ou Infringentes');
        $fnCadastrar(51, 'Emenda', 'Podendo ser complementado: Constitucional; de Comisso; de Bancada; de Relatoria');
        $fnCadastrar(52, 'Escala', 'Podendo ser complementado: de Frias');
        $fnCadastrar(53, 'Escritura', 'Podendo ser complementado: Pblica; de Imvel');
        $fnCadastrar(54, 'Estatuto', 'Podendo ser complementado: Social');
        $fnCadastrar(55, 'Exposio de Motivos', 'Espcie prpria');
        $fnCadastrar(56, 'Extrato', 'Podendo ser complementado: de Sistemas; Bancrio');
        $fnCadastrar(57, 'Fatura', 'Espcie prpria');
        $fnCadastrar(58, 'Ficha', 'Podendo ser complementado: de Cadastro; de Inscrio');
        $fnCadastrar(59, 'Fluxograma', 'Podendo ser complementado: de Processo; de Documentos; de Blocos');
        $fnCadastrar(60, 'Folha', 'Podendo ser complementado: de Frequncia de Estagirio; de Frequncia de Servidor');
        $fnCadastrar(61, 'Folheto/Folder', 'Podendo ser complementado: de Evento');
        $fnCadastrar(62, 'Formulrio', 'Podendo ser complementado: de Contato; de Reviso');
        $fnCadastrar(63, 'Grade Curricular', 'Podendo ser complementado: do Curso');
        $fnCadastrar(64, 'Guia', 'Podendo ser complementado: de Recolhimento da Unio');
        $fnCadastrar(65, 'Histrico', 'Podendo ser complementado: Escolar');
        $fnCadastrar(66, 'Indicao', 'Espcie prpria utilizada pelo Poder Legislativo');
        $fnCadastrar(67, 'Informe', 'Como Documento Externo pode ser complementado: de Rendimentos');
        $fnCadastrar(68, 'Instruo', 'Podendo ser complementado: Normativa');
        $fnCadastrar(69, 'Inventrio', 'Podendo ser complementado: de Estoque; Extrajudicial; Judicial; em Cartrio');
        $fnCadastrar(70, 'Laudo', 'Podendo ser complementado: Mdico; Conclusivo');
        $fnCadastrar(71, 'Lei', 'Podendo ser complementado: Complementar');
        $fnCadastrar(72, 'Lista/Listagem', 'Podendo ser complementado: de Presena');
        $fnCadastrar(73, 'Livro', 'Podendo ser complementado: Caixa');
        $fnCadastrar(74, 'Mandado', 'Podendo ser complementado: de Busca e Apreenso; de Citao; de Intimao');
        $fnCadastrar(75, 'Manifesto', 'Espcie prpria');
        $fnCadastrar(76, 'Manual', 'Podendo ser complementado: do Usurio; do Sistema; do Equipamento');
        $fnCadastrar(77, 'Mapa', 'Podendo ser complementado: de Ruas; de Risco');
        $fnCadastrar(78, 'Medida Provisria', 'Espcie prpria');
        $fnCadastrar(79, 'Memorando', 'Como Documento Externo pode ser complementado: de Entendimento');
        $fnCadastrar(80, 'Memorando-circular', 'Mesma definio do Memorando com apenas uma diferena:  encaminhado simultaneamente a mais de um cargo.');
        $fnCadastrar(81, 'Memorial', 'Podendo ser complementado: Descritivo; de Incorporao');
        $fnCadastrar(82, 'Mensagem', 'Podendo ser complementado: de Aniversrio; de Boas Vindas');
        $fnCadastrar(83, 'Minuta', 'Podendo ser complementado: de Portaria; de Resoluo');
        $fnCadastrar(84, 'Moo', 'Podendo ser complementado: de Apoio; de Pesar; de Repdio');
        $fnCadastrar(85, 'Norma', 'Podendo ser complementado: Tcnica; de Conduta');
        $fnCadastrar(86, 'Nota', 'Podendo ser complementado: Tcnica; de Empenho');
        $fnCadastrar(87, 'Notificao', 'Podendo ser complementado: de Lanamento');
        $fnCadastrar(88, 'Ofcio', 'Modalidades de comunicao oficial.  expedido para e pelas autoridades. Tem como finalidade o tratamento de assuntos oficiais pelos rgos da Administrao Pblica entre si e tambm com particulares.');
        $fnCadastrar(89, 'Ofcio-Circular', 'Espcie prpria');
        $fnCadastrar(90, 'Oramento', 'Podendo ser complementado: de Obra; de Servio');
        $fnCadastrar(91, 'Ordem', 'Podendo ser complementado: de Servio; de Compra; do Dia');
        $fnCadastrar(92, 'Organograma', 'Podendo ser complementado: da Empresa');
        $fnCadastrar(93, 'Orientao', 'Podendo ser complementado: Normativa; Jurisprudencial');
        $fnCadastrar(94, 'Panfleto', 'Podendo ser complementado: de Promoo; de Evento');
        $fnCadastrar(95, 'Parecer', 'Tipo de Documento prprio da AGU e outros rgos pblicos.');
        $fnCadastrar(96, 'Passaporte', 'Espcie prpria');
        $fnCadastrar(97, 'Pauta', 'Podendo ser complementado: de Julgamentos; de Audincias; das Sees');
        $fnCadastrar(98, 'Petio', 'Podendo ser complementado: Inicial; Incidental');
        $fnCadastrar(99, 'Planilha', 'Podendo ser complementado: de Custos e Formao de Preos');
        $fnCadastrar(100, 'Plano', 'Podendo ser complementado: de Servio; de Contas Contbil');
        $fnCadastrar(101, 'Planta', 'Podendo ser complementado: Baixa; de Localizao; de Situao');
        $fnCadastrar(102, 'Portaria', 'Expressa deciso relativa a assuntos de interesse interno da Agncia.');
        $fnCadastrar(103, 'Precatrio', 'Podendo ser complementado: Alimentar; Federal; Estadual; Municipal');
        $fnCadastrar(104, 'Processo', 'Processo');
        $fnCadastrar(105, 'Procurao', 'Espcie prpria');
        $fnCadastrar(106, 'Programa', 'Podendo ser complementado: de Governo; de Melhoria');
        $fnCadastrar(107, 'Projeto', 'Podendo ser complementado: Tcnico; Comercial');
        $fnCadastrar(108, 'Pronturio', 'Podendo ser complementado: Mdico; Odontolgico');
        $fnCadastrar(109, 'Pronunciamento', 'Espcie prpria');
        $fnCadastrar(110, 'Proposta', 'Podendo ser complementado: Comercial; de Oramento; Tcnica');
        $fnCadastrar(111, 'Prospecto', 'Podendo ser complementado: de Fundos');
        $fnCadastrar(112, 'Protocolo', 'Podendo ser complementado: de Entendimentos; de Entrega');
        $fnCadastrar(113, 'Prova', 'Podendo ser complementado: de Conceito; de Proficincia');
        $fnCadastrar(114, 'Questionrio', 'Podendo ser complementado: de Avaliao; de Pesquisa; Socioeconmico');
        $fnCadastrar(115, 'Receita', 'Espcie prpria');
        $fnCadastrar(116, 'Recibo', 'Podendo ser complementado: de Pagamento; de Entrega');
        $fnCadastrar(117, 'Recurso', 'Podendo ser complementado: Administrativo; Judicial');
        $fnCadastrar(118, 'Regimento', 'Podendo ser complementado: Interno');
        $fnCadastrar(119, 'Registro', 'Podendo ser complementado: de Detalhes de Chamadas - CDR; de Acesso; Comercial');
        $fnCadastrar(120, 'Regulamento', 'Podendo ser complementado: Geral; Disciplinar; de Administrao');
        $fnCadastrar(121, 'Relao', 'Podendo ser complementado: de Bens Reversveis - RBR');
        $fnCadastrar(122, 'Relatrio', 'Podendo ser complementado: de Conformidade; de Medies; de Prestao de Contas; de Viagem a Servio; Fotogrfico; Tcnico');
        $fnCadastrar(123, 'Release', 'Podendo ser complementado: de Resultados; de Produtos; de Servios');
        $fnCadastrar(124, 'Representao', 'Podendo ser complementado: Comercial; Processual; Fiscal');
        $fnCadastrar(125, 'Requerimento', 'Podendo ser complementado: Administrativo; de Adaptao; de Alterao Tcnica; de Alterao Tcnica; de Autocadastramento de Estao; de Licenciamento de Estao; de Servio de Telecomunicaes');
        $fnCadastrar(126, 'Requisio', 'Podendo ser complementado: de Auditoria; de Excluso; de Segunda Via');
        $fnCadastrar(127, 'Resoluo', 'Expressa deciso quanto ao provimento normativo que regula a implementao da poltica de telecomunicaes brasileira, a prestao dos servios de telecomunicaes, a administrao dos recursos  prestao e o funcionamento da Agncia.');
        $fnCadastrar(128, 'Resumo', 'Podendo ser complementado: Tcnico');
        $fnCadastrar(129, 'Roteiro', 'Podendo ser complementado: de Instalao; de Inspeo');
        $fnCadastrar(130, 'Sentena', 'Podendo ser complementado: de Mrito; Terminativa; Declaratria; Constitutiva; Condenatria; Mandamental; Executiva');
        $fnCadastrar(131, 'Sinopse', 'Podendo ser complementado: do Livro; do Estudo Tcnico');
        $fnCadastrar(132, 'Solicitao', 'Podendo ser complementado: de Pagamento');
        $fnCadastrar(133, 'Smula', 'Expressa deciso quanto  interpretao da legislao de telecomunicaes e fixa entendimento sobre matrias de competncia da Agncia, com efeito vinculativo.');
        $fnCadastrar(134, 'Tabela', 'Podendo ser complementado: de Visto; de Passaporte; de Certido');
        $fnCadastrar(135, 'Telegrama', 'Espcie prpria');
        $fnCadastrar(136, 'Termo', 'Podendo ser complementado: de Opo por Auxlio Financeiro; de Opo para Contribuio ao CPSS; de Conciliao; de Devoluo; de Doao; de Recebimento; de Resciso; de Compromisso de Estgio; de Representao; de Responsabilidade de Instalao - TRI');
        $fnCadastrar(137, 'Tese', 'Podendo ser complementado: de Doutorado');
        $fnCadastrar(138, 'Testamento', 'Podendo ser complementado: Particular; Vital; Cerrado; Conjuntivo');
        $fnCadastrar(139, 'Ttulo', 'Podendo ser complementado: de Eleitor; Pblico; de Capitalizao');
        $fnCadastrar(140, 'Voto', 'Espcie prpria');
        $fnCadastrar(141, 'Carteira', 'Podendo ser complementado: Nacional de Habilitao');
        $fnCadastrar(142, 'Carto', 'Podendo ser complementado: de Identificao');
        $fnCadastrar(143, 'CPF/CIC', 'Espcie prpria');
        $fnCadastrar(144, 'CNPJ', 'Espcie prpria');
        $fnCadastrar(145, 'Calendrio', 'Podendo ser complementado: de Reunies');
        $fnCadastrar(146, 'CNH', 'CNH');
        $fnCadastrar(147, 'RG', 'RG');
        $fnCadastrar(148, 'Agenda', 'Podendo ser complementado: de Reunio');
        $fnCadastrar(149, 'Anlise', 'Como Documento Externo pode ser complementado: Contbil');
        $fnCadastrar(150, 'Anotao', 'Podendo ser complementado: de Responsabilidade Tcnica - ART');
        $fnCadastrar(151, 'udio', 'Podendo ser complementado: de Reunio');
        $fnCadastrar(152, 'Boleto', 'Podendo ser complementado: de Pagamento; de Cobrana; de Cobrana Registrada; de Cobrana sem Registro');
        $fnCadastrar(153, 'Conta', 'Podendo ser complementado: Telefnica; de gua; de Luz');
        $fnCadastrar(154, 'Contrarrazes', 'Podendo ser complementado: em Recurso; em Apelao; em Embargos Infringentes');
        $fnCadastrar(155, 'Correspondncia', 'Espcie prpria');
        $fnCadastrar(156, 'Cota', 'Tipo de Documento prprio da AGU.');
        $fnCadastrar(157, 'Credencial', 'Podendo ser complementado: de Segurana; de Agente de Fiscalizao');
        $fnCadastrar(158, 'Croqui', 'Podendo ser complementado: de Acesso, Urbano');
        $fnCadastrar(159, 'Defesa', 'Podendo ser complementado: Administrativa; Judicial');
        $fnCadastrar(160, 'Demonstrao', 'Podendo ser complementado: de Resultado do Exerccio - DRE; de Fluxo de Caixa; Financeira; Contbil');
        $fnCadastrar(161, 'Denncia', 'Espcie prpria');
        $fnCadastrar(162, 'Esclarecimento', 'Espcie prpria utilizada em Licitao $fnCadastrar(ComprasNet);');
        $fnCadastrar(163, 'Escriturao', 'Podendo ser complementado: Contbil Digital - ECD; Fiscal Digital - EFD; Fiscal Digital - EFD-Contribuies');
        $fnCadastrar(164, 'Estratgia', 'Podendo ser complementado: da Contratao');
        $fnCadastrar(165, 'Impugnao', 'Espcie prpria utilizada em Licitao $fnCadastrar(ComprasNet);');
        $fnCadastrar(166, 'Informao', 'Tipo de Documento prprio da AGU.');
        $fnCadastrar(167, 'Inteno', 'Podendo ser complementado: de Recurso; de Compra; de Venda');
        $fnCadastrar(168, 'Licena', 'Podendo ser complementado: de Estao');
        $fnCadastrar(169, 'Matria', 'Podendo ser complementado: para Apreciao');
        $fnCadastrar(170, 'Material', 'Podendo ser complementado: Publicitrio; de Evento; de Promoo');
        $fnCadastrar(171, 'Memria', 'Podendo ser complementado: de Clculo');
        $fnCadastrar(172, 'Movimentao', 'Podendo ser complementado: de Bens Mveis');
        $fnCadastrar(173, 'Pedido', 'Podendo ser complementado: de Reconsiderao; de Esclarecimento');
        $fnCadastrar(174, 'Reclamao', 'Espcie prpria');
        $fnCadastrar(175, 'Referendo', 'Espcie prpria');
        $fnCadastrar(176, 'Resultado', 'Podendo ser complementado: de Exame Mdico; de Contestao');
        $fnCadastrar(177, 'Vdeo', 'Podendo ser complementado: de Reunio');


        //----------------------------------------------------------------------
        // Tarefas
        //----------------------------------------------------------------------
        $objDTO = new TarefaDTO();

        $fnCadastrar = function($strNome = '', $strHistoricoCompleto = 'N', $strHistoricoCompleto = 'N', $strFecharAndamentosAbertos = 'N', $strLancarAndamentoFechado = 'N', $strPermiteProcessoFechado = 'N', $strIdTarefaModulo = '') use($objDTO, $objBD) {

            $objDTO->unSetTodos();
            $objDTO->setStrIdTarefaModulo($strIdTarefaModulo);

            if ($objBD->contar($objDTO) == 0) {

                $objUltimaTarefaDTO = new TarefaDTO();
                $objUltimaTarefaDTO->retNumIdTarefa();
                $objUltimaTarefaDTO->setNumMaxRegistrosRetorno(1);
                $objUltimaTarefaDTO->setOrd('IdTarefa', InfraDTO::$TIPO_ORDENACAO_DESC);
                $objUltimaTarefaDTO = $objBD->consultar($objUltimaTarefaDTO);

                $objDTO->setNumIdTarefa($objUltimaTarefaDTO->getNumIdTarefa() + 1);
                $objDTO->setStrNome($strNome);
                $objDTO->setStrSinHistoricoResumido($strHistoricoCompleto);
                $objDTO->setStrSinHistoricoCompleto($strHistoricoCompleto);
                $objDTO->setStrSinFecharAndamentosAbertos($strFecharAndamentosAbertos);
                $objDTO->setStrSinLancarAndamentoFechado($strLancarAndamentoFechado);
                $objDTO->setStrSinPermiteProcessoFechado($strPermiteProcessoFechado);
                $objDTO->setStrIdTarefaModulo($strIdTarefaModulo);
                $objBD->cadastrar($objDTO);
            }
        };

        //TODO: Corrigir mensagem com portugus errado
        $fnCadastrar('Processo trmitado externamente para a entidade @UNIDADE_DESTINO@ - @REPOSITORIO_DESTINO@ (@PROCESSO@, @UNIDADE@, @USUARIO@)', 'S', 'S', 'N', 'S', 'N', 'PEN_PROCESSO_EXPEDIDO');
        $fnCadastrar('Processo recebido da entidade @ENTIDADE_ORIGEM@ - @REPOSITORIO_ORIGEM@ (@PROCESSO@, @ENTIDADE_ORIGEM@, @UNIDADE_DESTINO@, @USUARIO@)', 'S', 'S', 'N', 'S', 'N', 'PEN_PROCESSO_RECEBIDO');
        $fnCadastrar('O processo foi recusado pelo orgo @UNIDADE_DESTINO@ pelo seguinte motivo: @MOTIVO@', 'S', 'S', 'N', 'N', 'S', 'PEN_PROCESSO_RECUSADO');
        $fnCadastrar('Trmite externo do processo cancelado em @DATA_HORA@ pelo Usurio @USUARIO@', 'S', 'S', 'N', 'S', 'N', 'PEN_PROCESSO_CANCELADO');
        $fnCadastrar('Operacao externa de @OPERACAO@ registrada em @DATA_HORA@ (@PESSOA_IDENTIFICACAO@ - @PESSOA_NOME@)\n @COMPLEMENTO@', 'S', 'S', 'S', 'S', 'N', 'PEN_OPERACAO_EXTERNA');

        //----------------------------------------------------------------------
        // Operaes por Tarefas
        //----------------------------------------------------------------------
        $objDTO = new RelTarefaOperacaoDTO();

        $fnCadastrar = function($strCodigoOperacao, $numIdTarefa) use($objDTO, $objBD) {

            $objDTO->unSetTodos();
            $objDTO->setStrCodigoOperacao($strCodigoOperacao);
            $objDTO->setNumIdTarefa($numIdTarefa);

            if ($objBD->contar($objDTO) == 0) {
                $objBD->cadastrar($objDTO);
            }
        };

        //$fnCadastrar("01", 0);// Registro (Padro);
        $fnCadastrar("02", 32); //  Envio de documento avulso/processo ($TI_PROCESSO_REMETIDO_UNIDADE = 32;);
        $fnCadastrar("03", 51); //  Cancelamento/exclusao ou envio de documento ($TI_CANCELAMENTO_DOCUMENTO = 51;);
        $fnCadastrar("04", 13); //  Recebimento de documento ($TI_RECEBIMENTO_DOCUMENTO = 13;);
        $fnCadastrar("05", 1); // Autuacao ($TI_GERACAO_PROCEDIMENTO = 1;);
        $fnCadastrar("06", 101); // Juntada por anexacao ($TI_ANEXADO_PROCESSO = 101;);
        //$fnCadastrar("07", 0);// Juntada por apensacao;
        //$fnCadastrar("08", 0);// Desapensacao;
        $fnCadastrar("09", 24); //  Arquivamento ($TI_ARQUIVAMENTO = 24;);
        //$fnCadastrar("10", 0);// Arquivamento no Arquivo Nacional;
        //$fnCadastrar("11", 0);// Eliminacao;
        //$fnCadastrar("12", 0);// Sinistro;
        //$fnCadastrar("13", 0);// Reconstituicao de processo;
        $fnCadastrar("14", 26); // Desarquivamento ($TI_DESARQUIVAMENTO = 26;);
        //$fnCadastrar("15", 0);// Desmembramento;
        //$fnCadastrar("16", 0);// Desentranhamento;
        //$fnCadastrar("17", 0);// Encerramento/abertura de volume no processo;
        //$fnCadastrar("18", 0);// Registro de extravio;

        $objDTO = new InfraAgendamentoTarefaDTO();

        $fnCadastrar = function($strComando, $strDesc) use($objDTO, $objBD, $objRN) {

            $objDTO->unSetTodos();
            $objDTO->setStrComando($strComando);

            if ($objBD->contar($objDTO) == 0) {

                $objDTO->setStrDescricao($strDesc);
                $objDTO->setStrStaPeriodicidadeExecucao('D');
                $objDTO->setStrPeriodicidadeComplemento('0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23');
                $objDTO->setStrSinAtivo('S');
                $objDTO->setStrSinSucesso('S');

                $objBD->cadastrar($objDTO);
            }
        };

        $fnCadastrar('PENAgendamentoRN::seiVerificarServicosBarramento', 'Verificao dos servios de fila de processamento esto em execuo');


        /* ---------- antigo mtodo (instalarV002R003S000US024) ---------- */

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_tramite_processado',
            'cols' => array(
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'dth_ultimo_processamento' => array($objMetaBD->tipoDataHora(), PenMetaBD::NNULLO),
                'numero_tentativas' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'sin_recebimento_concluido' => array($objMetaBD->tipoTextoFixo(1), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('id_tramite')),
        ));

        $objInfraParametro = new InfraParametro($objInfraBanco);
        $objInfraParametro->setValor('PEN_NUMERO_TENTATIVAS_TRAMITE_RECEBIMENTO', '3');


        /* ---------- antigo mtodo (instalarV002R003S000IW001) ---------- */

        $objDTO = new TarefaDTO();
        $objBD = new TarefaBD($objInfraBanco);

        $fnAlterar = function($strIdTarefaModulo, $strNome) use($objDTO, $objBD) {

            $objDTO->unSetTodos();
            $objDTO->setStrIdTarefaModulo($strIdTarefaModulo);
            $objDTO->setNumMaxRegistrosRetorno(1);
            $objDTO->retStrNome();
            $objDTO->retNumIdTarefa();

            $objDTO = $objBD->consultar($objDTO);

            if (empty($objDTO)) {

                $objDTO->setStrNome($strNome);
                $objBD->cadastrar($objDTO);
            } else {

                $objDTO->setStrNome($strNome);
                $objBD->alterar($objDTO);
            }
        };

        $fnAlterar('PEN_PROCESSO_RECEBIDO', 'Processo recebido da entidade @ENTIDADE_ORIGEM@ - @REPOSITORIO_ORIGEM@');

        /* ---------- antigo mtodo (instalarV002R003S001US035) ---------- */
        $objMetaBanco = $this->inicializarObjMetaBanco();

        if (!$objMetaBanco->isColunaExiste('md_pen_tramite_processado', 'tipo_tramite_processo')) {
            $objMetaBanco->adicionarColuna('md_pen_tramite_processado', 'tipo_tramite_processo', 'CHAR(2)', PenMetaBD::NNULLO);
            $objMetaBanco->adicionarValorPadraoParaColuna('md_pen_tramite_processado', 'tipo_tramite_processo', 'RP');
        }

        if ($objMetaBanco->isChaveExiste('md_pen_tramite_processado', 'pk_md_pen_tramite_processado')) {

            $objMetaBanco->removerChavePrimaria('md_pen_tramite_processado', 'pk_md_pen_tramite_processado');
            $objMetaBanco->adicionarChavePrimaria('md_pen_tramite_processado', 'pk_md_pen_tramite_processado', array('id_tramite', 'tipo_tramite_processo'));
        }

        /* ---------- antigo mtodo (instalarV003R003S003IW001) ---------- */

        //----------------------------------------------------------------------
        // Novas sequncias
        //----------------------------------------------------------------------
        $objInfraSequencia = new InfraSequencia($objInfraBanco);

        if (!$objInfraSequencia->verificarSequencia('md_pen_rel_doc_map_enviado')) {
            $objInfraSequencia->criarSequencia('md_pen_rel_doc_map_enviado', '1', '1', '9999999999');
        }

        if (!$objInfraSequencia->verificarSequencia('md_pen_rel_doc_map_recebido')) {
            $objInfraSequencia->criarSequencia('md_pen_rel_doc_map_recebido', '1', '1', '9999999999');
        }

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_rel_doc_map_enviado',
            'cols' => array(
                'id_mapeamento' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'codigo_especie' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'id_serie' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'sin_padrao' => array($objMetaBD->tipoTextoFixo(1), 'S')
            ),
            'pk' => array('cols'=>array('id_mapeamento')),
            'fks' => array(
                'serie' => array('nome' => 'fk_md_pen_rel_doc_map_env_seri', 'cols' => array('id_serie', 'id_serie')),
                'md_pen_especie_documental' => array('nome' => 'fk_md_pen_rel_doc_map_env_espe', 'cols' => array('id_especie', 'codigo_especie')),
            )
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_rel_doc_map_recebido',
            'cols' => array(
                'id_mapeamento' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'codigo_especie' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'id_serie' => array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'sin_padrao' => array($objMetaBD->tipoTextoFixo(1), 'S')
            ),
            'pk' => array('cols'=>array('id_mapeamento')),
            'fks' => array(
                'serie' => array('nome' => 'fk_md_pen_rel_doc_map_rec_seri', 'cols' => array('id_serie', 'id_serie')),
                'md_pen_especie_documental' => array('nome' => 'fk_md_pen_rel_doc_map_rec_espe', 'cols' => array('id_especie', 'codigo_especie')),
            )
        ));

        $objBD = new GenericoBD($objInfraBanco);
        if ($objMetaBD->isTabelaExiste('md_pen_rel_tipo_documento_mapeamento_recebido')) {
            $objDTO = new PenRelTipoDocMapRecebidoDTO();
            $fnCadastrar = function($numCodigoEspecie, $numIdSerie) use($objDTO, $objBD) {
                $objDTO->unSetTodos();
                $objDTO->setNumCodigoEspecie($numCodigoEspecie);
                $objDTO->setNumIdSerie($numIdSerie);

                if ($objBD->contar($objDTO) == 0) {

                    $objDTO->setStrPadrao('S');
                    $objBD->cadastrar($objDTO);
                }
            };

            $arrDados = $objInfraBanco->consultarSql('SELECT DISTINCT codigo_especie, id_serie FROM md_pen_rel_tipo_documento_mapeamento_recebido');
            if (!empty($arrDados)) {
                foreach ($arrDados as $arrDocMapRecebido) {
                    $fnCadastrar($arrDocMapRecebido['codigo_especie'], $arrDocMapRecebido['id_serie']);
                }
            }

            $objMetaBD->removerTabela('md_pen_rel_tipo_documento_mapeamento_recebido');
        }

        if ($objMetaBD->isTabelaExiste('md_pen_rel_serie_especie')) {
            $objDTO = new PenRelTipoDocMapEnviadoDTO();
            $fnCadastrar = function($numCodigoEspecie, $numIdSerie) use($objDTO, $objBD) {

                $objDTO->unSetTodos();
                $objDTO->setNumCodigoEspecie($numCodigoEspecie);
                $objDTO->setNumIdSerie($numIdSerie);

                if ($objBD->contar($objDTO) == 0) {
                    $objDTO->setStrPadrao('S');
                    $objBD->cadastrar($objDTO);
                }
            };

            $arrDados = $objInfraBanco->consultarSql('SELECT DISTINCT codigo_especie, id_serie FROM md_pen_rel_serie_especie');
            if (!empty($arrDados)) {
                foreach ($arrDados as $arrDocMapEnviado) {
                    $fnCadastrar($arrDocMapEnviado['codigo_especie'], $arrDocMapEnviado['id_serie']);
                }
            }

            $objMetaBD->removerTabela('md_pen_rel_serie_especie');
        }


        /* ---------- antigo mtodo (instalarV004R003S003IW002) ---------- */
        $strTipo = $this->inicializarObjMetaBanco()->tipoTextoGrande();
        $objMetaBanco->adicionarColuna('md_pen_recibo_tramite', 'cadeia_certificado_temp', $strTipo, PenMetaBD::SNULLO);
        BancoSEI::getInstance()->executarSql("update md_pen_recibo_tramite set cadeia_certificado_temp = cadeia_certificado");
        $objMetaBanco->excluirColuna('md_pen_recibo_tramite', 'cadeia_certificado');
        $objMetaBanco->renomearColuna('md_pen_recibo_tramite', 'cadeia_certificado_temp', 'cadeia_certificado', $strTipo);

        $objMetaBanco->adicionarColuna('md_pen_recibo_tramite_enviado', 'cadeia_certificado_temp', $strTipo, PenMetaBD::SNULLO);
        BancoSEI::getInstance()->executarSql("update md_pen_recibo_tramite_enviado set cadeia_certificado_temp = cadeia_certificado");
        $objMetaBanco->excluirColuna('md_pen_recibo_tramite_enviado', 'cadeia_certificado');
        $objMetaBanco->renomearColuna('md_pen_recibo_tramite_enviado', 'cadeia_certificado_temp', 'cadeia_certificado', $strTipo);

        /* ---------- antigo mtodo (instalarV005R003S005IW018) ---------- */
        $objBD = new GenericoBD(BancoSEI::getInstance());
        $objDTO = new TarefaDTO();

        $fnCadastrar = function($strNome = '', $strHistoricoCompleto = 'N', $strHistoricoCompleto = 'N', $strFecharAndamentosAbertos = 'N', $strLancarAndamentoFechado = 'N', $strPermiteProcessoFechado = 'N', $strIdTarefaModulo = '') use($objDTO, $objBD) {

            $objDTO->unSetTodos();
            $objDTO->setStrIdTarefaModulo($strIdTarefaModulo);

            if ($objBD->contar($objDTO) == 0) {

                $objUltimaTarefaDTO = new TarefaDTO();
                $objUltimaTarefaDTO->retNumIdTarefa();
                $objUltimaTarefaDTO->setNumMaxRegistrosRetorno(1);
                $objUltimaTarefaDTO->setOrd('IdTarefa', InfraDTO::$TIPO_ORDENACAO_DESC);
                $objUltimaTarefaDTO = $objBD->consultar($objUltimaTarefaDTO);

                $objDTO->setNumIdTarefa($objUltimaTarefaDTO->getNumIdTarefa() + 1);
                $objDTO->setStrNome($strNome);
                $objDTO->setStrSinHistoricoResumido($strHistoricoCompleto);
                $objDTO->setStrSinHistoricoCompleto($strHistoricoCompleto);
                $objDTO->setStrSinFecharAndamentosAbertos($strFecharAndamentosAbertos);
                $objDTO->setStrSinLancarAndamentoFechado($strLancarAndamentoFechado);
                $objDTO->setStrSinPermiteProcessoFechado($strPermiteProcessoFechado);
                $objDTO->setStrIdTarefaModulo($strIdTarefaModulo);
                $objBD->cadastrar($objDTO);
            }
        };

        $fnCadastrar('O trmite externo do processo foi abortado manualmente devido a falha no trmite', 'S', 'S', 'N', 'N', 'S', 'PEN_EXPEDICAO_PROCESSO_ABORTADA');

        /* ---------- antigo mtodo (instalarV005R003S005IW023) ---------- */
        $objBD = new GenericoBD(BancoSEI::getInstance());

        $objDTO = new TarefaDTO();
        $objDTO->retNumIdTarefa();
        $objDTO->retStrNome();

        $fnAtualizar = function($strIdTarefaModulo, $strNome) use($objDTO, $objBD) {

            $objDTO->unSetTodos();
            $objDTO->setStrIdTarefaModulo($strIdTarefaModulo);

            $objTarefaDTO = $objBD->consultar($objDTO);

            if (!empty($objTarefaDTO)) {

                $objTarefaDTO->setStrNome($strNome);

                $objBD->alterar($objTarefaDTO);
            }
        };
        // Tramitao externa do processo @processo@ concluda com sucesso. Recebido na @UnidadeDestino@ - @hierarquia_superior@ -@repositrio_de_estruturas@
        $fnAtualizar('PEN_PROCESSO_EXPEDIDO', 'Processo em tramitao externa para @UNIDADE_DESTINO@ - @UNIDADE_DESTINO_HIRARQUIA@ - @REPOSITORIO_DESTINO@');
        $fnAtualizar('PEN_PROCESSO_RECEBIDO', 'Processo recebido da unidade externa @ENTIDADE_ORIGEM@ - @ENTIDADE_ORIGEM_HIRARQUIA@ - @REPOSITORIO_ORIGEM@');
        $fnAtualizar('PEN_OPERACAO_EXTERNA', 'Tramitao externa do processo @PROTOCOLO_FORMATADO@ concluda com sucesso. Recebido em @UNIDADE_DESTINO@ - @UNIDADE_DESTINO_HIRARQUIA@ - @REPOSITORIO_DESTINO@');

        /* ---------- antigo mtodo (instalarV006R004S004WI001) ---------- */
        $objInfraParametro = new InfraParametro(BancoSEI::getInstance());
        $objInfraParametro->setValor('PEN_TAMANHO_MAXIMO_DOCUMENTO_EXPEDIDO', 50);

        /* ---------- antigo mtodo (instalarV007R004S005WI002) ---------- */

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_recibo_tramite_hash',
            'cols' => array(
                'id_tramite_hash' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'numero_registro' => array($objMetaBD->tipoTextoFixo(16), PenMetaBD::NNULLO),
                'id_tramite' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'tipo_recibo' => array($objMetaBD->tipoTextoFixo(1), PenMetaBD::NNULLO),
                'hash_componente_digital ' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('id_tramite_hash')),
            'fks' => array(
                'md_pen_tramite' => array('nome' => 'fk_md_pen_rec_tram_hash_tram', 'cols' => array(array('numero_registro', 'id_tramite'), array('numero_registro', 'id_tramite')))
            )
        ));

        $objMetaBD->adicionarColuna('md_pen_recibo_tramite_recebido', 'cadeia_certificado', $this->inicializarObjMetaBanco()->tipoTextoGrande(), PenMetaBD::SNULLO);

        $objInfraSequencia = new InfraSequencia(BancoSEI::getInstance());
        if (!$objInfraSequencia->verificarSequencia('md_pen_recibo_tramite_hash')) {
            $objInfraSequencia->criarSequencia('md_pen_recibo_tramite_hash', '1', '1', '9999999999');
        }

        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO_ANTIGO);
        $objInfraParametroDTO->setStrValor('1.0.0');

        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroBD->cadastrar($objInfraParametroDTO);

        $this->logar(' EXECUTADA A INSTALACAO DA VERSAO 0.0.1 DO MODULO PEN NO SEI COM SUCESSO');
    }

    /* Contm atualizaes da versao 1.0.1 do modulo */
    protected function instalarV101() {
        /* ---------- antigo mtodo (instalarV008R004S006IW003) ---------- */
        $objBD = new GenericoBD(BancoSEI::getInstance());

        $objTarefaDTO = new TarefaDTO();
        $objTarefaDTO->setStrIdTarefaModulo('PEN_PROCESSO_RECEBIDO');
        $objTarefaDTO->retNumIdTarefa();

        $objTarefaDTO = $objBD->consultar($objTarefaDTO);

        $objTarefaDTO->setStrSinLancarAndamentoFechado('N');
        $objTarefaDTO->setStrSinPermiteProcessoFechado('S');

        $objBD->alterar($objTarefaDTO);

        /* ---------- antigo mtodo (instalarV006R004S001US039) ---------- */
        $objMetaBD = $this->inicializarObjMetaBanco();
        $objInfraBanco = BancoSEI::getInstance();

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_hipotese_legal',
            'cols' => array(
                'id_hipotese_legal'=> array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'nome' => array($objMetaBD->tipoTextoVariavel(255), PenMetaBD::NNULLO),
                'sin_ativo' => array($objMetaBD->tipoTextoFixo(1), 'S'),
            ),
            'pk' => array('cols'=>array('id_hipotese_legal')),
        ));

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_rel_hipotese_legal',
            'cols' => array(
                'id_mapeamento' => array($objMetaBD->tipoNumeroGrande(), PenMetaBD::NNULLO),
                'id_hipotese_legal'=> array($objMetaBD->tipoNumero(), PenMetaBD::NNULLO),
                'id_hipotese_legal_pen'=> array($objMetaBD->tipoNumero(), PenMetaBD::SNULLO),
                'tipo' => array($objMetaBD->tipoTextoFixo(1), 'E'),
                'sin_ativo' => array($objMetaBD->tipoTextoFixo(1), 'S'),
            ),
            'pk' => array('cols'=>array('id_mapeamento')),
            'fks' => array(
                'hipotese_legal' => array('nome' => 'fk_md_pen_rel_hipotese_legal', 'cols' => array('id_hipotese_legal', 'id_hipotese_legal')),
                'md_pen_hipotese_legal' => array('nome' => 'fk_md_pen_rel_hipotese_pen', 'cols' => array('id_hipotese_legal', 'id_hipotese_legal_pen'))
            )
        ));

        $objInfraSequencia = new InfraSequencia($objInfraBanco);

        if(!$objInfraSequencia->verificarSequencia('md_pen_hipotese_legal')){
            $objInfraSequencia->criarSequencia('md_pen_hipotese_legal', '1', '1', '9999999999');
        }

        if(!$objInfraSequencia->verificarSequencia('md_pen_rel_hipotese_legal')){
            $objInfraSequencia->criarSequencia('md_pen_rel_hipotese_legal', '1', '1', '9999999999');
        }

        $objHipoteseLegalDTO = new HipoteseLegalDTO();
        $objHipoteseLegalDTO->setDistinct(true);
        $objHipoteseLegalDTO->setStrStaNivelAcesso(1);
        $objHipoteseLegalDTO->setOrdStrNome(InfraDTO::$TIPO_ORDENACAO_ASC);
        $objHipoteseLegalDTO->retNumIdHipoteseLegal();
        $objHipoteseLegalDTO->retStrNome();

        $objMetaBD = $this->inicializarObjMetaBanco();

        $objMetaBD->criarTabela(array(
            'tabela' => 'md_pen_parametro',
            'cols' => array(
                'nome'=> array($objMetaBD->tipoTextoVariavel(100), PenMetaBD::NNULLO),
                'valor' => array($objMetaBD->tipoTextoGrande(), PenMetaBD::NNULLO)
            ),
            'pk' => array('cols'=>array('nome')),
        ));

        //Agendamento
        $objDTO = new InfraAgendamentoTarefaDTO();

        $fnCadastrar = function($strComando, $strDesc) use($objDTO, $objBD, $objRN) {

            $objDTO->unSetTodos();
            $objDTO->setStrComando($strComando);

            if ($objBD->contar($objDTO) == 0) {

                $objDTO->setStrDescricao($strDesc);
                $objDTO->setStrStaPeriodicidadeExecucao('D');
                $objDTO->setStrPeriodicidadeComplemento('0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23');
                $objDTO->setStrSinAtivo('S');
                $objDTO->setStrSinSucesso('S');

                $objBD->cadastrar($objDTO);
            }
        };

        $fnCadastrar('PENAgendamentoRN::atualizarHipotesesLegais', 'Verificao se h novas hipteses legais do barramento.');

        /* altera o parmetro da verso de banco */
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO_ANTIGO);
        $objInfraParametroDTO->setStrValor('1.0.1');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.0 do modulo */
    protected function instalarV110() {
        $objMetaBD = $this->objMeta;

        //Adiciona a coluna de indentificao nas hipteses que vem do barramento
        $objMetaBD->adicionarColuna('md_pen_hipotese_legal', 'identificacao', $this->inicializarObjMetaBanco()->tipoNumero(), PenMetaBD::SNULLO);

        //Adiciona a coluna de descricao nos parmetros
        $objMetaBD->adicionarColuna('md_pen_parametro', 'descricao', $this->inicializarObjMetaBanco()->tipoTextoVariavel(255), PenMetaBD::SNULLO);

        //Cria os parmetros do mdulo PEN barramento (md_pen_parametro [ nome, valor ])
        $this->criarParametro('PEN_ENDERECO_WEBSERVICE', 'https://pen-api.trafficmanager.net/interoperabilidade/soap/v2/', 'Endereo do Web Service');
        $this->criarParametro('PEN_ENDERECO_WEBSERVICE_PENDENCIAS', 'https://pen-pendencias.trafficmanager.net/', 'Endereo do Web Service de Pendncias');
        $this->criarParametro('PEN_ENVIA_EMAIL_NOTIFICACAO_RECEBIMENTO', 'N', 'Envia E-mail de Notificao de Recebimento');
        $this->criarParametro('PEN_ID_REPOSITORIO_ORIGEM', '1', 'ID do Repositrio de Origem');
        $this->criarParametro('PEN_LOCALIZACAO_CERTIFICADO_DIGITAL', '/opt/sei/web/modulos/mod-sei-barramento/CONITall.pem', 'Localizao do Certificado Digital');
        $this->criarParametro('PEN_NUMERO_TENTATIVAS_TRAMITE_RECEBIMENTO', '3', 'Nmero Mximo de Tentativas de Recebimento');
        $this->criarParametro('PEN_SENHA_CERTIFICADO_DIGITAL', '1234', 'Senha do Certificado Digital');
        $this->criarParametro('PEN_TAMANHO_MAXIMO_DOCUMENTO_EXPEDIDO', '50', 'Tamanho Mximo de Documento Expedido');
        $this->criarParametro('PEN_TIPO_PROCESSO_EXTERNO', '100000320', 'Tipo de Processo Externo');
        $this->criarParametro('PEN_UNIDADE_GERADORA_DOCUMENTO_RECEBIDO', '110000001', 'Unidade Geradora de Processo e Documento Recebido');

        //Deleta os parmetros do infra_parametros
        $this->deletaParametroInfra('PEN_ENDERECO_WEBSERVICE');
        $this->deletaParametroInfra('PEN_ENDERECO_WEBSERVICE_PENDENCIAS');
        $this->deletaParametroInfra('PEN_ENVIA_EMAIL_NOTIFICACAO_RECEBIMENTO');
        $this->deletaParametroInfra('PEN_ID_REPOSITORIO_ORIGEM');
        $this->deletaParametroInfra('PEN_LOCALIZACAO_CERTIFICADO_DIGITAL');
        $this->deletaParametroInfra('PEN_NUMERO_TENTATIVAS_TRAMITE_RECEBIMENTO');
        $this->deletaParametroInfra('PEN_SENHA_CERTIFICADO_DIGITAL');
        $this->deletaParametroInfra('PEN_TAMANHO_MAXIMO_DOCUMENTO_EXPEDIDO');
        $this->deletaParametroInfra('PEN_TIPO_PROCESSO_EXTERNO', '100000320');
        $this->deletaParametroInfra('PEN_UNIDADE_GERADORA_DOCUMENTO_RECEBIDO');

        //Alterar nomeclatura do recurso
        $objDTO = new PenParametroDTO();
        $objDTO->setStrNome('HIPOTESE_LEGAL_PADRAO');
        $objDTO->retStrNome();
        $objBD = new PenParametroBD(BancoSEI::getInstance());
        $objDTO = $objBD->consultar($objDTO);
        if ($objDTO) {
            $objDTO->setStrDescricao('Hiptese Legal Padro');
            $objBD->alterar($objDTO);
        } else {
            $objDTO = new PenParametroDTO();
            $objDTO->setStrNome('HIPOTESE_LEGAL_PADRAO');
            $objDTO->setStrValor(1);
            $objDTO->setStrDescricao('Hiptese Legal Padro');
            $objBD->cadastrar($objDTO);
        }

        /* altera o parmetro da verso de banco */
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO_ANTIGO);
        $objInfraParametroDTO->setStrValor('1.1.0');
        $objInfraParametroBD->alterar($objInfraParametroDTO);

    }

    /* Contm atualizaes da versao 1.1.1 do mdulo */
    protected function instalarV111() {

        //Ajuste em nome da varivel de verso do mdulo VERSAO_MODULO_PEN
        BancoSEI::getInstance()->executarSql("update infra_parametro set nome = '" . self::PARAMETRO_VERSAO_MODULO . "' where nome = '" . self::PARAMETRO_VERSAO_MODULO_ANTIGO . "'");

        /* altera o parmetro da verso de banco */
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.1');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.7 do mdulo */
    protected function instalarV117() {

        /* Cadastramento de novas espcies documentais */
        $objEspecieDocumentalBD = new GenericoBD(BancoSEI::getInstance());
        $objEspecieDocumentalDTO = new EspecieDocumentalDTO();

        $fnCadastrar = function($dblIdEspecie, $strNomeEspecie, $strDescricao) use($objEspecieDocumentalDTO, $objEspecieDocumentalBD) {
            $objEspecieDocumentalDTO->unSetTodos();
            $objEspecieDocumentalDTO->setDblIdEspecie($dblIdEspecie);
            if ($objEspecieDocumentalBD->contar($objEspecieDocumentalDTO) == 0) {
                $objEspecieDocumentalDTO->setStrNomeEspecie($strNomeEspecie);
                $objEspecieDocumentalDTO->setStrDescricao($strDescricao);
                $objEspecieDocumentalBD->cadastrar($objEspecieDocumentalDTO);
            }
        };

        $fnCadastrar(178, 'Alegaes', 'Muito comum no Judicirio, tendo previso no CPC. Podendo ser complementado "Finais", o que representaria o documento "Alegaes Finais".');
        $fnCadastrar(179, 'Anexo', 'Documento ou processo juntado em carter definitivo a outro processo, para dar continuidade a uma ao administrativa.');
        $fnCadastrar(180, 'Documento', 'Informao registrada, qualquer que seja o suporte ou formato, que no est reunida e ordenada em processo.');
        $fnCadastrar(181, 'Apartado', 'Apartado por si s, autos apartados ou partado sigiloso.');
        $fnCadastrar(182, 'Apresentao', 'Documentos que so apresentaes propriamente ditas.');
        $fnCadastrar(183, 'Diagnstico', 'Diagnstico mdico, auditoria, etc.');
        $fnCadastrar(184, 'Exame', 'Exame laboratorial, mdico, etc.');
        $fnCadastrar(185, 'Pgina', 'Pgina do Dirio Oficial da Unio.');
        $fnCadastrar(186, 'Estudo', 'Podendo ser complementado com "Tcnico Preliminar da Contratao"; "Tcnico".');
        $fnCadastrar(999, 'Outra', 'Outras espcies documentais no identificadas.');

        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.7');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.8 do mdulo */
    protected function instalarV118() {

        $objInfraMetaBD = new InfraMetaBD(BancoSEI::getInstance());

        //Correo de chave primria para considerar campo de tipo de recibo
        $objInfraMetaBD->excluirChavePrimaria('md_pen_tramite_processado','pk_md_pen_tramite_processado');
        $objInfraMetaBD->adicionarChavePrimaria('md_pen_tramite_processado','pk_md_pen_tramite_processado',array('id_tramite','tipo_tramite_processo'));

        //Atribuio de dados da unidade de origem e destino no trmite
        $objInfraMetaBD->adicionarColuna('md_pen_tramite','id_repositorio_origem', $objInfraMetaBD->tipoNumero(16), 'null');
        $objInfraMetaBD->adicionarColuna('md_pen_tramite','id_estrutura_origem', $objInfraMetaBD->tipoNumero(16), 'null');
        $objInfraMetaBD->adicionarColuna('md_pen_tramite','id_repositorio_destino', $objInfraMetaBD->tipoNumero(16), 'null');
        $objInfraMetaBD->adicionarColuna('md_pen_tramite','id_estrutura_destino', $objInfraMetaBD->tipoNumero(16), 'null');

        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.8');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.9 do mdulo */
    protected function instalarV119() {
        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.9');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }


    /* Contm atualizaes da versao 1.1.10 do mdulo */
    protected function instalarV1110() {
        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.10');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.11 do mdulo */
    protected function instalarV1111() {
        BancoSEI::getInstance()->executarSql("DELETE FROM participante WHERE EXISTS (SELECT md_pen_processo_eletronico.id_procedimento FROM md_pen_processo_eletronico WHERE md_pen_processo_eletronico.id_procedimento = participante.id_protocolo AND participante.sta_participacao='R')");

        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.11');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }


    /* Contm atualizaes da versao 1.1.12 do mdulo */
    protected function instalarV1112() {
        $objInfraMetaBD = new InfraMetaBD(BancoSEI::getInstance());

        //[#22] Correo de erro de consistncia no recebimento de processos com concorrncia
        $objInfraMetaBD->adicionarColuna('md_pen_tramite','sta_tipo_tramite', $objInfraMetaBD->tipoTextoFixo(1), 'null');
        $objInfraMetaBD->alterarColuna('md_pen_procedimento_andamento','id_procedimento',$objInfraMetaBD->tipoNumeroGrande(),'null');
        $objInfraMetaBD->adicionarColuna('md_pen_procedimento_andamento','numero_registro', $objInfraMetaBD->tipoTextoFixo(16), 'null');

        $objTramiteDTO = new TramiteDTO();
        $objTramiteDTO->retNumIdTramite();
        $objTramiteDTO->retStrNumeroRegistro();

        $objTramiteRN = new TramiteBD(BancoSEI::getInstance());
        $arrObjTramiteDTO = $objTramiteRN->listar($objTramiteDTO);

        foreach ($arrObjTramiteDTO as $objTramiteDTO) {
            $objProcedimentoAndamentoDTO = new ProcedimentoAndamentoDTO();
            $objProcedimentoAndamentoDTO->retDblIdAndamento();
            $objProcedimentoAndamentoDTO->retStrNumeroRegistro();
            $objProcedimentoAndamentoDTO->retDblIdTramite();
            $objProcedimentoAndamentoDTO->setDblIdTramite($objTramiteDTO->getNumIdTramite());

            $objProcedimentoAndamentoBD = new ProcedimentoAndamentoBD(BancoSEI::getInstance());
            $arrObjProcedimentoAndamentoDTO = $objProcedimentoAndamentoBD->listar($objProcedimentoAndamentoDTO);
            foreach ($arrObjProcedimentoAndamentoDTO as $objProcedimentoAndamentoDTO) {

                $objProcedimentoAndamentoDTO->setStrNumeroRegistro($objTramiteDTO->getStrNumeroRegistro());
                $objProcedimentoAndamentoBD->alterar($objProcedimentoAndamentoDTO);
            }
        }

        //$objInfraMetaBD->alterarColuna('md_pen_procedimento_andamento','numero_registro', $objInfraMetaBD->tipoTextoFixo(16), 'not null');

        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.12');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.13 do mdulo */
    protected function instalarV1113() {

        //Fix-31 - Erro ao Configurar Campo 'numero_registro' como Not Null no Scritp de atualiza<E7><E3>o
        $objInfraMetaBD = new InfraMetaBD(BancoSEI::getInstance());
        $objInfraMetaBD->alterarColuna('md_pen_procedimento_andamento','numero_registro', $objInfraMetaBD->tipoTextoFixo(16), 'null');

        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.13');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.14 do mdulo */
    protected function instalarV1114()
    {
        $objInfraMetaBD = new InfraMetaBD(BancoSEI::getInstance());
        $objInfraSequencia = new InfraSequencia(BancoSEI::getInstance());

        SessaoSEI::getInstance(false)->simularLogin(SessaoSEI::$USUARIO_SEI, SessaoSEI::$UNIDADE_TESTE);
        SessaoInfra::setObjInfraSessao(SessaoSEI::getInstance());
        BancoInfra::setObjInfraIBanco(BancoSEI::getInstance());

        #[Fix-35] Correo de erro de integridade ao retornar mais de um elemento na consulta de mapeamento
        $objInfraMetaBD->criarIndice('md_pen_rel_doc_map_enviado', 'ak1_rel_doc_map_enviado', array('id_serie'), true);
        $objInfraMetaBD->criarIndice('md_pen_rel_doc_map_recebido', 'ak1_rel_doc_map_recebido', array('codigo_especie'), true);

        //30 - Correo de erros de chave duplicada devido a concorrncia de transaes
        $objInfraSequenciaRN = new InfraSequenciaRN();
        $objInfraSequenciaDTO = new InfraSequenciaDTO();

        //Sequncia: md_pen_seq_procedimento_andam
        $rs = BancoSEI::getInstance()->consultarSql('select max(id_andamento) as total from md_pen_procedimento_andamento');
        $numMaxId = $rs[0]['total'];
        if ($numMaxId==null){
            $numMaxId = 0;
        }
        BancoSEI::getInstance()->criarSequencialNativa('md_pen_seq_procedimento_andam', $numMaxId + 1);
        $objInfraSequenciaDTO->setStrNome('md_pen_procedimento_andamento');
        $objInfraSequenciaDTO->retStrNome();
        $arrObjInfraSequenciaDTO = $objInfraSequenciaRN->listar($objInfraSequenciaDTO);
        $objInfraSequenciaRN->excluir($arrObjInfraSequenciaDTO);


        //Sequncia: md_pen_seq_hipotese_legal
        $rs = BancoSEI::getInstance()->consultarSql('select max(id_hipotese_legal) as total from md_pen_hipotese_legal');
        $numMaxId = $rs[0]['total'];
        if ($numMaxId==null){
            $numMaxId = 0;
        }
        BancoSEI::getInstance()->criarSequencialNativa('md_pen_seq_hipotese_legal', $numMaxId + 1);
        $objInfraSequenciaDTO->setStrNome('md_pen_hipotese_legal');
        $objInfraSequenciaDTO->retStrNome();
        $arrObjInfraSequenciaDTO = $objInfraSequenciaRN->listar($objInfraSequenciaDTO);
        $objInfraSequenciaRN->excluir($arrObjInfraSequenciaDTO);


        //Sequncia: md_pen_seq_rel_hipotese_legal
        $rs = BancoSEI::getInstance()->consultarSql('select max(id_mapeamento) as total from md_pen_rel_hipotese_legal');
        $numMaxId = $rs[0]['total'];
        if ($numMaxId==null){
            $numMaxId = 0;
        }
        BancoSEI::getInstance()->criarSequencialNativa('md_pen_seq_rel_hipotese_legal', $numMaxId + 1);
        $objInfraSequenciaDTO->setStrNome('md_pen_rel_hipotese_legal');
        $objInfraSequenciaDTO->retStrNome();
        $arrObjInfraSequenciaDTO = $objInfraSequenciaRN->listar($objInfraSequenciaDTO);
        $objInfraSequenciaRN->excluir($arrObjInfraSequenciaDTO);


        //Sequncia: md_pen_seq_recibo_tramite_hash
        $rs = BancoSEI::getInstance()->consultarSql('select max(id_tramite_hash) as total from md_pen_recibo_tramite_hash');
        $numMaxId = $rs[0]['total'];
        if ($numMaxId==null){
            $numMaxId = 0;
        }
        BancoSEI::getInstance()->criarSequencialNativa('md_pen_seq_recibo_tramite_hash', $numMaxId + 1);
        $objInfraSequenciaDTO->setStrNome('md_pen_recibo_tramite_hash');
        $objInfraSequenciaDTO->retStrNome();
        $arrObjInfraSequenciaDTO = $objInfraSequenciaRN->listar($objInfraSequenciaDTO);
        $objInfraSequenciaRN->excluir($arrObjInfraSequenciaDTO);

        //Sequncia: md_pen_seq_rel_doc_map_enviado
        $rs = BancoSEI::getInstance()->consultarSql('select max(id_mapeamento) as total from md_pen_rel_doc_map_enviado');
        $numMaxId = $rs[0]['total'];
        if ($numMaxId==null){
            $numMaxId = 0;
        }
        BancoSEI::getInstance()->criarSequencialNativa('md_pen_seq_rel_doc_map_enviado', $numMaxId + 1);
        $objInfraSequenciaDTO->setStrNome('md_pen_rel_doc_map_enviado');
        $objInfraSequenciaDTO->retStrNome();
        $arrObjInfraSequenciaDTO = $objInfraSequenciaRN->listar($objInfraSequenciaDTO);
        $objInfraSequenciaRN->excluir($arrObjInfraSequenciaDTO);

        //Sequncia: md_pen_seq_rel_doc_map_recebid
        $rs = BancoSEI::getInstance()->consultarSql('select max(id_mapeamento) as total from md_pen_rel_doc_map_recebido');
        $numMaxId = $rs[0]['total'];
        if ($numMaxId==null){
            $numMaxId = 0;
        }
        BancoSEI::getInstance()->criarSequencialNativa('md_pen_seq_rel_doc_map_recebid', $numMaxId + 1);
        $objInfraSequenciaDTO->setStrNome('md_pen_rel_doc_map_recebido');
        $objInfraSequenciaDTO->retStrNome();
        $arrObjInfraSequenciaDTO = $objInfraSequenciaRN->listar($objInfraSequenciaDTO);
        $objInfraSequenciaRN->excluir($arrObjInfraSequenciaDTO);


        //Sequncia: md_pen_seq_tramite_pendente
        $rs = BancoSEI::getInstance()->consultarSql('select max(id) as total from md_pen_tramite_pendente');
        $numMaxId = $rs[0]['total'];
        if ($numMaxId==null){
            $numMaxId = 0;
        }
        BancoSEI::getInstance()->criarSequencialNativa('md_pen_seq_tramite_pendente', $numMaxId + 1);
        $objInfraSequenciaDTO->setStrNome('md_pen_tramite_pendente');
        $objInfraSequenciaDTO->retStrNome();
        $arrObjInfraSequenciaDTO = $objInfraSequenciaRN->listar($objInfraSequenciaDTO);
        $objInfraSequenciaRN->excluir($arrObjInfraSequenciaDTO);

        //Fix 28 - Erro Data too long for column 'nome' at row 1
        $objInfraMetaBD->alterarColuna('md_pen_componente_digital','nome', $objInfraMetaBD->tipoTextoVariavel(255), 'not null');

        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD($this->inicializarObjInfraIBanco());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.14');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }


    /* Contm atualizaes da versao 1.1.15 do mdulo */
    protected function instalarV1115() {

        //Fix-31 - Erro ao Configurar Campo 'numero_registro' como Not Null no Scritp de atualiza<E7><E3>o
        $objInfraMetaBD = new InfraMetaBD(BancoSEI::getInstance());
        $objInfraMetaBD->alterarColuna('md_pen_procedimento_andamento','numero_registro', $objInfraMetaBD->tipoTextoFixo(16), 'null');

        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.15');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.16 do mdulo */
    protected function instalarV1116() {

        //Fix-31 - Erro ao Configurar Campo 'numero_registro' como Not Null no Scritp de atualiza<E7><E3>o
        $objInfraMetaBD = new InfraMetaBD(BancoSEI::getInstance());
        $objInfraMetaBD->alterarColuna('md_pen_procedimento_andamento','numero_registro', $objInfraMetaBD->tipoTextoFixo(16), 'null');

        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.16');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.1.17 do mdulo */
    protected function instalarV1117() {

        // Definio de funo annima responsvel por realizar as seguintes tarefas:
        //  (1) Identificar a tarefa com ID conflitante do SEI
        //  (2) Criar nova tarefa identica mas com ID correto dentro das faixas definidas pelo SEI (maior que 1000)
        //  (3) Atualizar o id_tarefa de todas as atividades relacionadas
        //  (4) Remover a tarefa anterior com ID invlido
        //  (5) Atualizar o campo id_tarefa_modulo com o valor correspondente
        $fnCadastrar = function($numIdTarefa, $strIdTarefaModulo) {

            // Identificar a tarefa com ID conflitante do SEI
            $objTarefaRN = new TarefaRN();
            $objTarefaBD = new TarefaBD(BancoSEI::getInstance());
            $objTarefaDTOAntigo = new TarefaDTO();
            $objTarefaDTOAntigo->retTodos();
            $objTarefaDTOAntigo->setStrIdTarefaModulo($strIdTarefaModulo);
            $objTarefaDTOAntigo = $objTarefaBD->consultar($objTarefaDTOAntigo);

            if(isset($objTarefaDTOAntigo)) {
                try {
                    // Criar nova tarefa identica mas com ID correto dentro das faixas definidas pelo SEI (maior que 1000)
                    InfraDebug::getInstance()->gravar("Duplicando tarefa customizadas $strIdTarefaModulo utilizando o controle de sequncia 1000");
                    $objTarefaDTO = new TarefaDTO();
                    $objTarefaDTO->setNumIdTarefa($numIdTarefa);
                    $objTarefaDTO->setStrNome($objTarefaDTOAntigo->getStrNome());
                    $objTarefaDTO->setStrSinHistoricoResumido($objTarefaDTOAntigo->getStrSinHistoricoResumido());
                    $objTarefaDTO->setStrSinHistoricoCompleto($objTarefaDTOAntigo->getStrSinHistoricoCompleto());
                    $objTarefaDTO->setStrSinFecharAndamentosAbertos($objTarefaDTOAntigo->getStrSinFecharAndamentosAbertos());
                    $objTarefaDTO->setStrSinLancarAndamentoFechado($objTarefaDTOAntigo->getStrSinLancarAndamentoFechado());
                    $objTarefaDTO->setStrSinPermiteProcessoFechado($objTarefaDTOAntigo->getStrSinPermiteProcessoFechado());
                    $objTarefaDTO->setStrIdTarefaModulo(null);
                    $objTarefaBD->cadastrar($objTarefaDTO);

                    // Atualizar o id_tarefa de todas as atividades relacionadas
                    InfraDebug::getInstance()->gravar("Atualizando atividades com chave da nova tarefa $strIdTarefaModulo");
                    $numIdTarefaAnterior = $objTarefaDTOAntigo->getNumIdTarefa();
                    BancoSEI::getInstance()->executarSql("UPDATE atividade SET id_tarefa = $numIdTarefa where id_tarefa = $numIdTarefaAnterior");

                    // Remover a tarefa anterior com ID invlido
                    InfraDebug::getInstance()->gravar("Apagando a tarefa anterior $strIdTarefaModulo");
                    $objTarefaBD->excluir($objTarefaDTOAntigo);

                    // Atualizar o campo id_tarefa_modulo com o valor correspondente
                    $objTarefaDTOUpdate = new TarefaDTO();
                    $objTarefaDTOUpdate->setNumIdTarefa($numIdTarefa);
                    $objTarefaDTOUpdate->setStrIdTarefaModulo($strIdTarefaModulo);
                    $objTarefaBD->alterar($objTarefaDTOUpdate);

                } catch (Exception $e) {
                    throw new InfraException($e);
                }
            }
        };

        $rsTabelaTarefa = BancoSEI::getInstance()->consultarSql('select max(id_tarefa) as ultimo from tarefa');
        $numMaxId = $rsTabelaTarefa[0]['ultimo'];
        if (!isset($numMaxId) || $numMaxId < 1000){
            $numMaxId = 1000;
        }

        $fnCadastrar(++$numMaxId, 'PEN_PROCESSO_EXPEDIDO');
        $fnCadastrar(++$numMaxId, 'PEN_PROCESSO_RECEBIDO');
        $fnCadastrar(++$numMaxId, 'PEN_PROCESSO_RECUSADO');
        $fnCadastrar(++$numMaxId, 'PEN_PROCESSO_CANCELADO');
        $fnCadastrar(++$numMaxId, 'PEN_OPERACAO_EXTERNA');
        $fnCadastrar(++$numMaxId, 'PEN_EXPEDICAO_PROCESSO_ABORTADA');

        InfraDebug::getInstance()->gravar('Atualizando sequncia das tabelas do sistema');


        //Na verso 3.1.0 do SEI, houve uma mudana na rotina de atualizao das sequences do banco de dados,
        //deixando de se utilizar a classe VersaoRN para utilizar a nova classe ScriptRN.
        //Devido a esta mudana,  necessrio avaliar qual a atual verso do SEI executar a rotina correta

        //Normaliza o formato de nmero de verso considerando dois caracteres para cada item (3.1.0 -> 030100)
        $numVersaoAtualSEI = explode('.', SEI_VERSAO);
        $numVersaoAtualSEI = array_map(function($item){ return str_pad($item, 2, '0', STR_PAD_LEFT); }, $numVersaoAtualSEI);
        $numVersaoAtualSEI = intval(join($numVersaoAtualSEI));

        //Normaliza o formato de nmero de verso considerando dois caracteres para cada item (3.1.0 -> 030100)
        $numVersaoMudancaAtualizarSequencias = explode('.', '3.1.0');
        $numVersaoMudancaAtualizarSequencias = array_map(function($item){ return str_pad($item, 2, '0', STR_PAD_LEFT); }, $numVersaoMudancaAtualizarSequencias);
        $numVersaoMudancaAtualizarSequencias = intval(join($numVersaoMudancaAtualizarSequencias));

        if($numVersaoMudancaAtualizarSequencias <= $numVersaoAtualSEI){
            //Procedimento de atualizao de sequncias compatvel com SEI 3.1.X
            $objScriptRN = new ScriptRN();
            $objScriptRN->atualizarSequencias();
        } else {
            //Procedimento de atualizao de sequncias compatvel com SEI 3.0.X
            $objVersaoRN = new VersaoRN();
            $objVersaoRN->atualizarSequencias();
        }


        //Altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.1.17');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }


    /* Contm atualizaes da versao 1.2.0 do mdulo */
    protected function instalarV1200()
    {
        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.2.0');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.2.1 do mdulo */
    protected function instalarV1201()
    {
        //Fix-47 - Corrigir erro com mapeamento de espcies documentais da origem
        $objInfraMetaBD = new InfraMetaBD(BancoSEI::getInstance());
        $objInfraMetaBD->adicionarColuna('md_pen_componente_digital', 'codigo_especie', $objInfraMetaBD->tipoNumero(), 'null');
        $objInfraMetaBD->adicionarColuna('md_pen_componente_digital', 'nome_especie_produtor', $objInfraMetaBD->tipoTextoVariavel(255), 'null');


        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.2.1');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.2.2 do mdulo */
    protected function instalarV1202()
    {
        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.2.2');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }

    /* Contm atualizaes da versao 1.2.3 do mdulo */
    protected function instalarV1203()
    {
        //altera o parmetro da verso de banco
        $objInfraParametroBD = new InfraParametroBD(BancoSEI::getInstance());
        $objInfraParametroDTO = new InfraParametroDTO();
        $objInfraParametroDTO->setStrNome(self::PARAMETRO_VERSAO_MODULO);
        $objInfraParametroDTO->setStrValor('1.2.3');
        $objInfraParametroBD->alterar($objInfraParametroDTO);
    }
}
